clear all;
close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Gaussian mixture data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m = 500; % total size
n = 200; % training size
p = .5; % prior prob of class 1
m1 = binornd(n,p);
m0 = m - m1;

%generate data
mu0 = [-1 -1];
SIGMA0 = [1 0; 0 1];
x0 = mvnrnd(mu0,SIGMA0,m0);

mu1 = [1 1];
SIGMA1 = [1 0; 0 1];
x1 = mvnrnd(mu1,SIGMA1,m1);

data = [x0; x1];
labels = [-ones(m0,1); ones(m1,1)];

%randomly permute the data
p = randperm(m);
data = data(p, :);
labels = labels(p);

%divide data into training set and test set
train_data = data(1:n,:);
train_labels = labels(1:n);

test_data = data((n+1):m,:);
test_labels = labels((n+1):m);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set L2QP options. Type "help svmtrain" for details
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[n d] = size(train_data);
i0 = find(train_labels == -1);
n0 = length(i0);
i1 = find(train_labels == 1);
n1 = length(i1);

gamma = n0/n1;
sigma = 0.8;
k = 0;
tol = 10^(-6);

l2qp_opt = sprintf('-f %f -a %f -d %d -i %d -e %e', sigma, gamma, k, d, tol);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run L2QP method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model = svmtrain(train_labels, train_data, l2qp_opt);
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Visualize results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure

x = [-5:0.1:5];
y = [-5:0.1:5];
[X1 X2] = meshgrid(x, x);

x1 = model.SVs(:,1);
x2 = model.SVs(:,2);
nsv = model.totalSV;
alpha = model.sv_coef;

%build a classifier
dr = zeros(length(X1), length(X2));
for j = 1:nsv
    X = (X1 - x1(j)).^2 + (X2 -x2(j)).^2;
    dr = dr + alpha(j)/(2*pi*sigma^2)*exp(-X/(2*sigma^2));
end

scatter(train_data(i0,1), train_data(i0,2), 50, '+r');
hold on;
scatter(train_data(i1,1), train_data(i1,2), 50, '*b');

contour(X1, X2, dr, [0 0], '-k');
scatter(x1, x2, 100, 'ok');
% surf(X1, X2, dr);
axis([-5 5 -5 5]);
axis square
box on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Predict labels of newly observed data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[dummy_label, acc, dummy_decison] = svmpredict(test_labels, test_data, model);
pe = 1 - acc(1)/100;
